<?php

namespace Imoje\Payment;

use Exception;

/**
 * Class Api
 *
 * @package Imoje\Payment
 */
class Api
{

	const TRANSACTION = 'transaction';
	const MERCHANT = 'merchant';
	const TRANSACTION_TYPE_SALE = 'sale';

	/**
	 * @var array
	 */
	private static $serviceUrls = [
		Util::ENVIRONMENT_PRODUCTION => 'https://api.imoje.pl/v1',
		//Util::ENVIRONMENT_SANDBOX    => 'https://sandbox.api.imoje.pl', sandbox doesnt exist at this moment
	];

	/**
	 * @var string
	 */
	private $authorizationToken;

	/**
	 * @var string
	 */
	private $merchantId;

	/**
	 * @var string
	 */
	private $serviceId;

	/**
	 * Api constructor.
	 *
	 * @param string $authorizationToken
	 * @param string $merchantId
	 * @param string $serviceId
	 */
	public function __construct($authorizationToken, $merchantId, $serviceId)
	{

		$this->authorizationToken = $authorizationToken;
		$this->merchantId = $merchantId;
		$this->serviceId = $serviceId;
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	public static function getServiceUrl($environment)
	{

		if(isset(self::$serviceUrls[$environment])) {
			return self::$serviceUrls[$environment];
		}

		return '';
	}

	/**
	 * @param int    $amount
	 * @param string $currency
	 * @param string $orderId
	 * @param string $paymentMethod
	 * @param string $paymentMethodCode
	 * @param string $successReturnUrl
	 * @param string $failureReturnUrl
	 * @param string $customerFirstName
	 * @param string $customerLastName
	 * @param string $customerEmail
	 * @param string $clientIp
	 * @param string $type
	 *
	 * @return string
	 */
	public function prepareData(
		$amount,
		$currency,
		$orderId,
		$paymentMethod,
		$paymentMethodCode,
		$successReturnUrl,
		$failureReturnUrl,
		$customerFirstName,
		$customerLastName,
		$customerEmail,
		$type = 'sale',
		$clientIp = ''
	) {

		if(!$clientIp) {
			$clientIp = $_SERVER['REMOTE_ADDR'];
		}

		return json_encode([
			'type'              => $type,
			'serviceId'         => $this->serviceId,
			'amount'            => $amount,
			'currency'          => $currency,
			'orderId'           => (string) $orderId,
			'paymentMethod'     => $paymentMethod,
			'paymentMethodCode' => $paymentMethodCode,
			'successReturnUrl'  => $successReturnUrl,
			'failureReturnUrl'  => $failureReturnUrl,
			'clientIp'          => $clientIp,
			'customer'          => [
				'firstName' => $customerFirstName,
				'lastName'  => $customerLastName,
				'email'     => $customerEmail,
			],
		]);
	}

	/**
	 * @param string $body
	 *
	 * @return array|bool|string
	 */
	public function createTransaction($body)
	{
		return $this->call(
			$this->getTransactionCreateUrl(Util::ENVIRONMENT_PRODUCTION),
			Util::METHOD_REQUEST_POST,
			$body,
			true
		);
	}

	/**
	 * Creates full form with order data.
	 *
	 * @param array  $transaction
	 * @param string $submitValue
	 * @param string $submitClass
	 * @param string $submitStyle
	 *
	 * @return string
	 * @throws Exception
	 */
	public function buildOrderForm($transaction, $submitValue = '', $submitClass = '', $submitStyle = '')
	{

		if(!isset($transaction['body']['action'])) {

			throw new Exception(json_encode([
					'action' => 'apiBuildOrderForm',
					'error'  => 'Doesnt action exist',
				]
			));
		}

		return Util::createOrderForm(Util::parseStringToArray($transaction['body']['action']['contentBodyRaw']), $transaction['body']['action']['url'], $transaction['body']['action']['method'], $submitValue, $submitClass, $submitStyle);
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	private function getTransactionCreateUrl($environment)
	{

		$baseUrl = self::getServiceUrl($environment);

		if($baseUrl) {
			return $baseUrl . '/' . self::MERCHANT . '/' . $this->merchantId . '/' . self::TRANSACTION;
		}

		return '';
	}

	/**
	 * @param string $url
	 * @param string $methodRequest
	 * @param string $body
	 * @param bool   $decode
	 *
	 * @return bool|string|array
	 */
	private function call($url, $methodRequest, $body, $decode = false)
	{

		$curl = curl_init($url);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $methodRequest);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curl, CURLOPT_TIMEOUT, 10);
		curl_setopt($curl, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Authorization: Bearer ' . $this->authorizationToken,
		]);

		$resultCurl = curl_exec($curl);

		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		if(($httpCode !== 200) || !$resultCurl) {
			return [
				'success' => false,
				'data'    => [
					'httpCode' => $httpCode,
					'error'    => curl_error($curl),
				],
			];
		}

		if($decode) {
			return [
				'success' => true,
				'body'    => json_decode($resultCurl, true),
			];
		}

		return [
			'success' => true,
			'body'    => $resultCurl,
		];
	}
}
